﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace FrequencyConverterMonitor
{
    /// <summary>
    /// Логика взаимодействия для BitComboBox.xaml
    /// </summary>
    public partial class BitComboBox : UserControl
    {
        public BitComboBox()
        {
            InitializeComponent();
            MainControl.ItemsSource = ItemsSource;
        }

        public void Add(SelectableObject item)
        {
            ItemsSource.Add(item);
        }

        public ObservableCollection<SelectableObject> ItemsSource { get; set; } = new ObservableCollection<SelectableObject>();

        private void OnCbObjectCheckBoxChecked(object sender, RoutedEventArgs e)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendFormat("{0}: ", Value);
            foreach (var cbObject in ItemsSource)
            {
                if (cbObject.IsSelected)
                    sb.AppendFormat("{0}, ", cbObject.Text);
            }
            string text = sb.ToString().Trim().TrimEnd(',');
            MainControl.Text = text;
        }

        private void OnCbObjectsSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ComboBox comboBox = (ComboBox)sender;
            comboBox.SelectedItem = null;
            SelectionChanged?.Invoke(this, EventArgs.Empty);
        }

        public event EventHandler SelectionChanged;

        public int Value
        {
            get
            {
                int value = 0;
                foreach (var item in ItemsSource)
                {
                    if (item.IsSelected)
                        value = value | (1 << item.BitNumber);
                }
                return value;
            }
            set
            {
                foreach (var item in ItemsSource)
                {
                    if ((value & (1 << item.BitNumber)) != 0)
                    {
                        item.IsSelected = true;
                    }
                    else
                    {
                        item.IsSelected = false;
                    }
                }
                OnCbObjectCheckBoxChecked(null, null);
            }
        }

        private void MainControl_DropDownClosed(object sender, EventArgs e)
        {
            OnCbObjectCheckBoxChecked(null, null);
        }
    }

    public class SelectableObject: INotifyPropertyChanged
    {
        bool _isSelected = false;
        public bool IsSelected
        {
            get { return _isSelected; }
            set
            {
                if (_isSelected != value)
                {
                    _isSelected = value;
                    PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(IsSelected)));
                }
            }
        }
        public int BitNumber { get { return (int)Value.Value; } }

        public string Text
        {
            get
            {
                return Value.Text;
            }
        }

        public string Text2
        {
            get
            {
                return "Бит " + BitNumber + " : " + Value.Text;
            }
        }
        public ParamValue Value { get; set; }

        public SelectableObject(ParamValue pValue)
        {
            Value = pValue;
        }

        public SelectableObject(ParamValue pValue, bool isSelected)
        {
            IsSelected = isSelected;
            Value = pValue;
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}
