﻿using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace FrequencyConverterMonitor
{
    /// <summary>
    /// Логика взаимодействия для ComPortWindow.xaml
    /// </summary>
    public partial class ComPortWindow : Window
    {
        public ComPortWindow()
        {
            InitializeComponent();
            InitComboBoxs();
        }

        public bool ShowDialog(Window owner, SerialPortSettings setting)
        {
            Owner = owner;

            this.cPort.Text = setting.Port;
            this.cBaudRate.SelectedItem = setting.Speed;
            this.cDataBits.SelectedItem = setting.DataBits;
            this.cStopBits.SelectedItem = setting.StopBits;
            this.cParity.SelectedItem = setting.Parity;
            this.cHandshake.SelectedItem = setting.HandShake;

            if (this.ShowDialog() == true)
            {
                lock (setting)
                {
                    setting.Port = this.cPort.Text;
                    setting.Speed = (int)this.cBaudRate.SelectedItem;
                    setting.DataBits = (int)this.cDataBits.SelectedItem;
                    setting.StopBits = (StopBits)this.cStopBits.SelectedItem;
                    setting.Parity = (Parity)this.cParity.SelectedItem;
                    setting.HandShake = (Handshake)this.cHandshake.SelectedItem;
                }
                return true;
            }
            else
            {
                return false;
            }
        }

        public void InitComboBoxs()
        {
            this.cPort.Items.Clear();
            foreach (var s in SerialPort.GetPortNames()) this.cPort.Items.Add(s);
            if (this.cPort.Items.Count > 0) this.cPort.SelectedIndex = 0;

            this.cBaudRate.Items.Clear();
            foreach (var s in new int[] { 300, 600, 1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200, }) this.cBaudRate.Items.Add(s);
            this.cBaudRate.SelectedItem = 9600;

            this.cDataBits.Items.Clear();
            foreach (var s in new int[] { 5, 6, 7, 8, }) this.cDataBits.Items.Add(s);
            this.cDataBits.SelectedItem = 8;
            FillComboBox<Parity>(cParity, Parity.None);
            FillComboBox<StopBits>(cStopBits, StopBits.Two);

            FillComboBox<Handshake>(cHandshake, Handshake.None);
        }

        void FillComboBox<T>(ComboBox cBox, T defaultValue)
        {
            Array arr = Enum.GetValues(typeof(T));
            foreach (object obj in arr)
            {
                cBox.Items.Add(obj);
            }
            cBox.SelectedItem = defaultValue;
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            this.DialogResult = true;
        }
    }
}
