﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Xml;
using System.IO;
using FrequencyConverterMonitor;
using System.Windows.Controls;
using System.Windows.Media;
using System.Globalization;

namespace FrequencyConverterMonitor
{
    public class ProjectInfo
    {
        public string FileName;
        public MainWindow Owner;
        List<DeviceInfo> _devices = new List<DeviceInfo>();
        public IEnumerable<DeviceInfo> Devices { get { return _devices; } }

        public void Add(DeviceInfo info)
        {
            _devices.Add(info);
        }

        public void Remove(DeviceInfo info)
        {
            _devices.Remove(info);
        }

        public void Save()
        {
            if (_isLoad) return;
            try
            {
                XmlDocument doc = new XmlDocument();
                doc.LoadXml("<?xml version=\"1.0\" encoding=\"utf-8\" ?>\r\n<project/>");
                foreach (var d in _devices)
                {
                    XmlElement dXml = doc.CreateElement("device");
                    dXml.SetAttribute("name", d.Name);
                    dXml.SetAttribute("deviceNumber", d.DeviceNumber.ToString());
                    dXml.SetAttribute("active", d.Active.ToString());
                    dXml.SetAttribute("showSlide", d.IsShow.ToString());
                    dXml.SetAttribute("showSetting", d.IsShowSetting.ToString());
                    dXml.SetAttribute("configName", d.Config.Name);
                    dXml.SetAttribute("configFileName", d.Config.FileName);
                    doc.DocumentElement.AppendChild(dXml);
                }
                doc.Save(FileName);
            }
            catch { }
        }

        bool _isLoad = false;

        public void Load(string pName)
        {
            FileName = pName;
            Clear();
            _isLoad = true;
            try
            {
                XmlDocument doc = new XmlDocument();
                doc.Load(FileName);
                foreach (XmlElement dXml in doc.GetElementsByTagName("device"))
                {
                    try
                    {
                        string name = dXml.GetAttribute("name");
                        int deviceNumber = int.Parse(dXml.GetAttribute("deviceNumber"));
                        string configName = dXml.GetAttribute("configName");
                        string configFileName = dXml.GetAttribute("configFileName");
                        bool active = dXml.GetAttribute("active").StartsWith("t", StringComparison.CurrentCultureIgnoreCase);
                        bool isShow = dXml.GetAttribute("showSlide").StartsWith("t", StringComparison.CurrentCultureIgnoreCase);
                        bool isShowSetting = dXml.GetAttribute("showSetting").StartsWith("t", StringComparison.CurrentCultureIgnoreCase);

                        DeviceInfo info = new DeviceInfo() { MainWindow = Owner };
                        var configs = DeviceConfig.GetTempldates();
                        DeviceConfig config = null;
                        foreach (var c in configs)
                        {
                            if (string.Equals(c.FileName, configFileName, StringComparison.CurrentCultureIgnoreCase))
                            {
                                config = c;
                                break;
                            }
                        }

                        if (config != null)
                        {
                            info.Init(name, deviceNumber, active, config);
                            Add(info);
                            if (isShow) info.Show();
                            else info.Hide();
                            //if (isShowSetting) info.ShowSettings();
                        }
                    }
                    catch { }
                }
            }
            catch { }
            _isLoad = false;
        }

        public void SaveAs(string pName)
        {
            FileName = pName;
            Save();
        }

        public void Clear()
        {
            _isLoad = true;
            try
            {
                foreach (var d in _devices.ToArray()) d.Close();
                _devices.Clear();
            }
            catch { }
            _isLoad = false;
        }
    }

    public class DeviceConfig
    {
        public string Name { get; set; }
        public string Type { get; set; }
        public string FileName { get; set; }

        public override string ToString()
        {
            return Name;
        }

        public static DeviceConfig[] GetTempldates()
        {
            List<DeviceConfig> list = new List<DeviceConfig>();
            string root = Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "templates");
            if (Directory.Exists(root))
            {
                foreach (var fileName in Directory.GetFiles(root, "*.xml"))
                {
                    try
                    {
                        DeviceConfig d = new DeviceConfig();
                        d.Load(fileName);
                        list.Add(d);
                    }
                    catch { }
                }
            }
            return list.ToArray();
        }

        public List<ParamGroup> Groups = new List<ParamGroup>();
        public List<ParamInfo> Params { get; set; } = new List<ParamInfo>();
        public Dictionary<string, ParamInfo> ParamNames = new Dictionary<string, ParamInfo>();

        public List<DeviceCommand> Commands = new List<DeviceCommand>();
        public List<RequestInfo> Reqeusts = new List<RequestInfo>();
        public List<SlideParam> MainParams = new List<SlideParam>();
        public List<SlideParam> SlideParams = new List<SlideParam>();
        public List<SlideParam> Inputs = new List<SlideParam>();
        public List<SlideParam> Outputs = new List<SlideParam>();
        public List<ErrorInfo> Errors = new List<ErrorInfo>();

        

        public bool Load(string fileName)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(fileName);
            FileName = Path.GetFileName(fileName);
            return Load(doc.DocumentElement);
        }

        public override bool Equals(object obj)
        {
            DeviceConfig info = obj as DeviceConfig;
            if (info == null) return false;
            if (string.IsNullOrEmpty(info.FileName) && string.IsNullOrEmpty(this.FileName))
            {
                if (string.Equals(info.Name, this.Name, StringComparison.InvariantCultureIgnoreCase)) return true;
            }
            else
            {
                if (string.Equals(info.FileName, this.FileName, StringComparison.InvariantCultureIgnoreCase)) return true;
            }
            return base.Equals(obj);
        }

        public override int GetHashCode()
        {
            string tmp = Name + "$" + FileName;
            return tmp.GetHashCode();
        }

        public bool Load(XmlElement xml)
        {
            Name = xml.GetAttribute("name");
            Type = xml.GetAttribute("type");

            var slide = xml["slide"];
            if (slide != null)
            {
                foreach (var cnode in slide.ChildNodes)
                {
                    XmlElement node = cnode as XmlElement;
                    if (node == null) continue;
                    switch (node.Name)
                    {
                        case "requests":
                            foreach (XmlElement xmlRequest in node.GetElementsByTagName("request"))
                            {
                                RequestInfo rInfo = new RequestInfo();
                                rInfo.ModbusFunction = xmlRequest.GetInt("modbusFunction", 3);
                                rInfo.RegNumber = xmlRequest.GetInt("regNumber");
                                rInfo.RegCount = xmlRequest.GetInt("regCount", 1);
                                Reqeusts.Add(rInfo);
                            }
                            break;
                        case "commands":
                            foreach (XmlElement xmlCommand in node.GetElementsByTagName("command"))
                            {
                                DeviceCommand rInfo = new DeviceCommand();
                                rInfo.Name = xmlCommand.GetAttribute("name");
                                if (Enum.TryParse<CommandType>(rInfo.Name, out CommandType t)) rInfo.Type = t;
                                rInfo.ModbusFunction = xmlCommand.GetInt("modbusFunction", 6);
                                rInfo.RegNumber = xmlCommand.GetInt("regNumber");
                                rInfo.Value = xmlCommand.GetInt("value");
                                Commands.Add(rInfo);
                            }
                            break;
                        case "mainParam":
                            MainParams.Add(SlideParam.FromXml(node));
                            break;
                        case "param":
                            SlideParams.Add(SlideParam.FromXml(node));
                            break;
                        case "inputs":
                            foreach (XmlElement xmlNode in node.GetElementsByTagName("item"))
                            {
                                Inputs.Add(SlideParam.FromXml(xmlNode));
                            }
                            break;
                        case "outputs":
                            foreach (XmlElement xmlNode in node.GetElementsByTagName("item"))
                            {
                                Outputs.Add(SlideParam.FromXml(xmlNode));
                            }
                            break;
                        case "errors":
                            foreach (XmlElement xmlNode in node.GetElementsByTagName("item"))
                            {
                                Errors.Add(ErrorInfo.FromXml(xmlNode));
                            }
                            break;
                    }
                }
            }

            var groups = xml["settings"];
            if (groups != null)
            {
                foreach (XmlElement groupXml in groups.GetElementsByTagName("group"))
                {
                    var g = new ParamGroup();
                    if (g.Load(this, groupXml))
                    {
                        Groups.Add(g);
                    }
                }
            }

            foreach (var p in Params) ParamNames.Add(p.Name,  p);

            foreach (var p in Params)
            {
                if (!string.IsNullOrEmpty(p.MinParamName)) p.MinParam = ParamNames[p.MinParamName];
                if (!string.IsNullOrEmpty(p.MaxParamName)) p.MaxParam = ParamNames[p.MaxParamName];
                if (!string.IsNullOrEmpty(p.KParamName)) p.KParam = ParamNames[p.KParamName];

                foreach (var info in p.IfInfo)
                {
                    if (!string.IsNullOrEmpty(info.Param))
                    {
                        if (info.Param != "type")
                        {
                            info.TestParam = ParamNames[info.Param];
                        }
                        else
                        {
                            info.IsTrue = string.Equals(info.Value, this.Type, StringComparison.InvariantCultureIgnoreCase);
                        }
                    }

                    if (!string.IsNullOrEmpty(info.MinParamName)) info.MinParam = ParamNames[info.MinParamName];
                    if (!string.IsNullOrEmpty(info.MaxParamName)) info.MaxParam = ParamNames[info.MaxParamName];
                    if (!string.IsNullOrEmpty(info.KParamName)) info.KParam = ParamNames[info.KParamName];
                }
            }

            return true;
        }
    }

    public enum CommandType
    {
        None,
        Run,
        Stop,
        Back,
        ErrorReset,
    }

    public class DeviceCommand
    {
        public CommandType Type;
        public string Name;
        public uint RegNumber;
        public uint ModbusFunction;
        public uint Value;
    }

    public class RequestInfo
    {
        public uint RegNumber;
        public uint RegCount;
        public uint ModbusFunction;
    }

    public class SlideParam: IParamInfo
    {
        public uint RegNumber;
        public uint ModbusFunction = 3;
        public uint WriteModbusFunction = 6;

        public uint? BitNumber= null;
        public ushort RegValue;
        public string ParamValue;

        public double DoubleValue
        {
            get
            {
                if (Sign)
                {
                    unchecked
                    {
                        return ((short)RegValue) * (K ?? 1);
                    }
                }
                return RegValue * (K ?? 1);
            }
        }

        public void SetErrorValue(uint modbusFunction, int address)
        {
            if (modbusFunction == ModbusFunction &&
                RegNumber == address)
            {
                ModbusValue = null;
                ParamValue = "???";
            }
        }

        public void UpdateValue(uint modbusFunction, int address, ushort data)
        {
            if (modbusFunction == ModbusFunction &&
                RegNumber == address)
            {
                lock (this)
                {
                    RegValue = data;

                    if (BitNumber.HasValue)
                    {
                        int tmp = data & (1 << (int)BitNumber.Value);
                        if (tmp != 0)
                        {
                            ParamValue = string.IsNullOrEmpty(TextOn) ? "On" : TextOn;
                        }
                        else
                        {
                            ParamValue = string.IsNullOrEmpty(TextOff) ? "Off" : TextOff;
                        }
                    }
                    else if (Values != null && Values.Count > 0)
                    {
                        var v = Values.Find(t => t.Value == RegValue);
                        if (v == null)
                        {
                            ParamValue = RegValue.ToString();
                        }
                        else
                        {
                            ParamValue = v.Text;
                        }
                    }
                    else
                    {
                        double v = (K ?? 1) * data;

                        if (Sign)
                        {
                            unchecked
                            {
                                v = (K ?? 1) * (short)data;
                            }
                        }

                        try
                        {
                            if (Format.StartsWith("X"))
                            {
                                ParamValue = ((int)v).ToString(Format);
                            }
                            else
                            {
                                ParamValue = v.ToString(Format);
                            }
                        }
                        catch
                        {
                            ParamValue = v.ToString();
                        }

                        if (!string.IsNullOrEmpty(Unit)) ParamValue += " " + Unit;
                    }
                }
            }
        }

        public string Name { get; set; }
        public string Caption { get; set; }
        public string Format { get; set; }
        public string Unit { get; set; }
        public double? K { get; set; } = null;
        public bool Sign { get; set; } = false;
        public int? ModbusValue { get; set; }
        public double? MinValue { get; set; } = 0.0;
        public double? MaxValue { get; set; } = 100.0;


        public string TextOff = null;
        public string TextOn = null;

        public TextBlock txCaption;
        public TextBlock txValue;


        public List<ParamValue> Values = null;

        public static SlideParam FromXml(XmlElement xml)
        {
            SlideParam info = new SlideParam();
            info.Name = xml.GetAttribute("name");

            info.ModbusFunction = xml.GetInt("modbusFunction", 3);
            info.WriteModbusFunction = xml.GetInt("writeModbusFunction", 6);

            info.RegNumber = xml.GetInt("regNumber");

            if (xml.HasAttribute("k")) info.K = xml.GetDouble("k");
            if (xml.HasAttribute("sign")) info.Sign = xml.GetAttribute("sign").StartsWith("t", StringComparison.InvariantCultureIgnoreCase);

            info.Caption = xml.GetAttribute("caption");
            info.Unit = xml.GetAttribute("unit");
            info.Format = xml.GetAttribute("format");

            if (xml.HasAttribute("bitNumber")) info.BitNumber = xml.GetInt("bitNumber");
            if (xml.HasAttribute("textOff")) info.TextOff = xml.GetAttribute("textOff");
            if (xml.HasAttribute("textOn")) info.TextOn = xml.GetAttribute("textOn");

            if (xml.HasAttribute("minValue")) info.MinValue = xml.GetDouble("minValue");
            if (xml.HasAttribute("maxValue")) info.MaxValue = xml.GetDouble("maxValue");

            if (xml.HasChildNodes)
            {
                foreach (XmlElement item in xml.GetElementsByTagName("item"))
                {
                    if (info.Values == null) info.Values = new List<ParamValue>();

                    ParamValue v = new ParamValue();
                    v.Text = item.GetAttribute("text");
                    v.Value = item.GetInt("value");
                    if (item.HasAttribute("k") || item.HasAttribute("K")) v.K = item.GetDouble("k");
                    if (item.HasAttribute("group")) v.Group = (int)item.GetInt("group", 1);
                    info.Values.Add(v);
                }
            }
            return info;
        }

        public void UpdateControl(ComminicationState state)
        {
            lock (this)
            {
                var c = txValue;
                if (c != null)
                {
                    if (state == ComminicationState.Normal)
                    {
                        c.Text = ParamValue;
                    }
                    else
                    {
                        c.Text = "???";
                    }
                }
            }
        }
    }

    public class InputOuputInfo
    {
        public int RegNumber { get; set; }
        public int ModbusFunction { get; set; }

        public string Name { get; set; }
        public string Caption { get; set; }
        public string Format { get; set; }
        public string ParamName { get; set; }


    }

    public class ErrorInfo
    {
        public uint RegNumber { get; set; }
        public uint ModbusFunction { get; set; }
        public bool Is { get; set; }
        public string Text { get; set; }
        public uint? Value { get; set; }
        public uint? Mask { get; set; }
        public uint? BitNumber = null;

        public void SetErrorValue(uint modbusFunction, int address)
        {
            if (modbusFunction == ModbusFunction &&
                RegNumber == address)
            {
                Is = false;
            }
        }
        internal void UpdateValue(uint modbusFunction, int address, ushort data)
        {
            if (modbusFunction == ModbusFunction &&
                RegNumber == address)
            {
                if (BitNumber.HasValue)
                {
                    int tmp = data & (1 << (int)BitNumber.Value);
                    Is = tmp != 0;
                }
                else if (Value.HasValue)
                {
                    Is = (data == Value.Value);
                }
                else if (Mask.HasValue)
                {
                    Is = ((data & Mask.Value) == Mask.Value);
                }
                else
                {
                    Is = false;
                }
            }
        }

        public static ErrorInfo FromXml(XmlElement xml)
        {
            ErrorInfo info = new ErrorInfo();
            info.ModbusFunction = xml.GetInt("modbusFunction", 3);
            info.RegNumber = xml.GetInt("regNumber");
            info.Text = xml.GetAttribute("text");
            if (xml.HasAttribute("value")) info.Value = xml.GetInt("value");
            if (xml.HasAttribute("mask")) info.Mask = xml.GetInt("mask");
            if (xml.HasAttribute("bitNumber")) info.BitNumber = xml.GetInt("bitNumber");
            return info;
        }
    }

    public class ParamInfoIf
    {
        public string MaxParamName;
        public string MinParamName;
        public string KParamName;

        public ParamInfo MaxParam;
        public ParamInfo MinParam;
        public ParamInfo KParam;

        public double? K = null;
        public double? MinValue = null;
        public double? MaxValue = null;
        public string Unit { get; set; }

        public string Param;
        public bool IsTrue = false;
        public ParamInfo TestParam;
        public string Value;

        public bool FromXml(XmlElement xml)
        {
            Param = xml.GetAttribute("param");
            Value = xml.GetAttribute("value");

            if (xml.HasAttribute("minValue")) MinValue = xml.GetDouble("minValue");
            if (xml.HasAttribute("maxValue")) MaxValue = xml.GetDouble("maxValue");
            if (xml.HasAttribute("k")) K = xml.GetDouble("k");
            Unit = xml.GetAttribute("unit");

            MaxParamName = xml.GetAttribute("maxParam");
            MinParamName = xml.GetAttribute("minParam");
            KParamName = xml.GetAttribute("kParam");

            return true;
        }
    }

    public interface IParamInfo
    {
        string Name { get; set; }
        string Caption { get; set; }

        double? K { get; set; }
        double? MinValue { get; set; }
        double? MaxValue { get; set; }
        string Unit { get; set; }
        bool Sign { get; set; }
        string Format { get; set; }

        int? ModbusValue { get; set; }
    }

    public class ParamInfo: IParamInfo
    {
        public int ModbusReadFunction = 3;
        public int ModbusWriteFunction = 6;
        public int ReadRegNumber = -1;
        public int WriteRegNumber = -1;

        public int RegCount = 1;

        public ParamType Type { get; set; }
        public int GroupCount { get; set; } = 1;

        //public UInt16? DeviceValue = null;
        //public UInt16? UseInputValue = null;
        public int? ModbusValue { get; set; }

        public ushort? ModbusValueU4
        {
            get
            {
                if (ModbusValue.HasValue) return unchecked((ushort)((ModbusValue ?? 0) & 0xFFFF));
                return null;
            }
        }
        public bool ReadOnly = false;

        public string Name { get; set; }
        public string Caption { get; set; }

        public override string ToString()
        {
            return Name + " <- " + ModbusValue;
        }


        public string MaxParamName;
        public string MinParamName;
        public string KParamName;

        public ParamInfo MaxParam;
        public ParamInfo MinParam;
        public ParamInfo KParam;
        public uint? NoWriteValue { get; set; } = null;

        //public object SourceValue { get; set; }
        //public object Value { get; set; }

        volatile private bool _changed = false;

        public bool IsChanged { get { return _changed; } }

        public void SetValue(ushort[] data, int index)
        {
            if (RegCount == 1)
            {
                _changed = true;

                if (data == null)
                {
                    ModbusValue = null;
                }
                else
                {
                    ModbusValue = data[index];
                    if (Sign) ModbusValue = unchecked((short)(ModbusValue.Value & 0xFFFF));

                    if (Type == ParamType.List)
                    {
                        var tmp = Values.Find(v => v.Value == ModbusValue);
                        if (tmp == null) StringValue = ModbusValue.ToString();
                        else StringValue = tmp.Text;
                    }
                    else //if (Type == ParamType.Value)
                    {
                        double v = ModbusValue.Value;
                        if (K.HasValue && K.Value != 0) v *= K.Value;
                        try
                        {
                            if (Format != null)
                            {
                                if (Format.StartsWith("X"))
                                {
                                    StringValue = ModbusValue?.ToString(Format);
                                }
                                else
                                {
                                    StringValue = v.ToString(Format);
                                }
                            }
                            StringValue = v.ToString();
                        }
                        catch
                        {
                            StringValue = v.ToString();
                        }
                    }
                }
            }
        }

        public string FormatValue(string f, double v)
        {
            if (string.IsNullOrWhiteSpace(f)) return v.ToString();
            try { return v.ToString(f, System.Globalization.CultureInfo.InvariantCulture); }
            catch { return v.ToString(); }
        }

        public void UpdateControl()
        {
            try
            {
                _isUpdateControl = true;

                if (Type == ParamType.List)
                {
                    var v = Values.Find(t => t.Value == ModbusValue);
                    if (v != null)
                    {
                        ComboBoxValueControl.SelectedItem = v;
                    }
                    else
                    {
                        ComboBoxValueControl.Text = StringValue;
                    }
                    //TextInputValue();
                    if (ReadOnly)
                    {
                        ComboBoxValueControl.IsReadOnly = true;
                        ComboBoxValueControl.IsEnabled = false;
                    }
                }
                else if (Type == ParamType.List2)
                {
                    if (ModbusValue.HasValue)
                    {
                        for (int i = 0; i < GroupCount; i++)
                        {
                            int b10 = (int)Math.Pow(10, i);
                            var control = ComboBoxValueControlList[i];
                            int gValue = (ModbusValue.Value / b10) % 10;
                            var v = Values.Find(t => t.Value == gValue && t.Group == b10);

                            if (v != null)
                            {
                                control.SelectedItem = v;
                            }
                            else
                            {
                                control.Text = gValue.ToString();
                            }
                        }
                    }
                    else
                    {
                        foreach (var c in ComboBoxValueControlList) c.SelectedItem = null;
                    }

                    if (ReadOnly)
                    {
                        foreach (var c in ComboBoxValueControlList)
                        {
                            c.IsReadOnly = true;
                            c.IsEnabled = false;
                        }
                    }
                }
                else if (Type == ParamType.BitList)
                {
                    BitControl.Value = this.ModbusValue ?? 0;
                    if (ReadOnly)
                    {
                        BitControl.IsEnabled = false;
                    }
                }
                else
                {
                    ValueControlEditor.UpdateParamInfo(true);
                    if (ReadOnly)
                    {
                        ValueControlEditor.IsReadOnly = true;
                    }
                }
            }
            finally
            {
                _changed = false;
                _isUpdateControl = false;
            }
        }

        public double? K { get; set; } = null;
        public double? MinValue { get; set; } = null;
        public double? MaxValue { get; set; } = null;

        public double? MinValueBase { get; set; } = null;
        public double? MaxValueBase { get; set; } = null;

        public string Unit { get; set; }
        public bool Sign { get; set; }
        public string Format { get; set; }
        

        public string StringValue { get; set; }


        public ParamGroup Group { get; set; }

        public List<ParamValue> Values { get; set; } = new List<ParamValue>();
        public List<ParamInfoIf> IfInfo = new List<ParamInfoIf>();

        public Border BorderCaptionControl;
        public Border BorderValueControl;
        //public TextBox TextValueControl;
        public NumericUpDown ValueControlEditor;
        public ComboBox ComboBoxValueControl;
        public BitComboBox BitControl;
        public List<ComboBox> ComboBoxValueControlList = new List<ComboBox>();

        bool _changeInfo = false;

        public bool IsChangeInfo => _changeInfo;

        int _inUpdateParamInfoCounter = 0;

        public void UpdateParamInfo()
        {
            if (_inUpdateParamInfoCounter > 1) return;
            try
            {
                _inUpdateParamInfoCounter++;

                foreach (var info in IfInfo)
                {
                    bool isTrue = false;

                    if (info.TestParam != null)
                    {
                        info.TestParam.UpdateParamInfo();
                        if (info.TestParam.ModbusValue.ToString() == info.Value)
                        {
                            _changeInfo = true;
                            isTrue = true;
                        }
                    }
                    else if (info.IsTrue)
                    {
                        isTrue = true;
                    }

                    if (isTrue)
                    {
                        if (info.Unit != null) this.Unit = info.Unit;

                        if (info.MinValue != null) this.MinValue = info.MinValue;
                        if (info.MaxValue != null) this.MaxValue = info.MaxValue;
                        if (info.K != null) this.K = info.K;

                        if (info.MinParam != null) this.MinParam = info.MinParam;
                        if (info.MaxParam != null) this.MaxParam = info.MaxParam;
                        if (info.KParam != null) this.KParam = info.KParam;
                    }
                }

                if (MinParam != null)
                {
                    MinParam.UpdateParamInfo();
                    MinValue = MinParam.DoubleValue;
                    _changeInfo = true;
                }

                if (MaxParam != null)
                {
                    MaxParam.UpdateParamInfo();
                    MaxValue = MaxParam.DoubleValue;
                    _changeInfo = true;
                }

                if (KParam != null)
                {
                    KParam.UpdateParamInfo();
                    K = KParam.DoubleValueK;
                    if (MinValueBase.HasValue) MinValue = MinValueBase.Value * K;
                    if (MaxValueBase.HasValue) MaxValue = MaxValueBase.Value * K;
                    _changeInfo = true;
                }
            }
            finally
            {
                _inUpdateParamInfoCounter--;
            }
        }

        public double DoubleValue
        {
            get
            {
                if (ModbusValue == null) return 0;
                if (Type == ParamType.List)
                {
                    var v = Values.Find(p => p.Value == ModbusValue);
                    if (v != null && v.K.HasValue) return v.K.Value;
                    return v.Value;
                }
                else if (K.HasValue) return ModbusValue.Value * K.Value;
                else return ModbusValue.Value;
            }
        }

        public double DoubleValueK
        {
            get
            {
                if (ModbusValue == null) return 1;
                if (Type == ParamType.List)
                {
                    var v = Values.Find(p => p.Value == ModbusValue);
                    if (v != null && v.K.HasValue) return v.K.Value;
                    return 1;
                }
                else if (K.HasValue) return ModbusValue.Value * K.Value;
                else return ModbusValue.Value;
            }
        }


        public bool Load(ParamGroup group, XmlElement xml)
        {
            if (group != null) this.Group = group;
            Name = xml.GetAttribute("name");
            Caption = xml.GetAttribute("text");

            if (xml.HasAttribute("minValue")) MinValue = xml.GetDouble("minValue");
            if (xml.HasAttribute("maxValue")) MaxValue = xml.GetDouble("maxValue");

            if (xml.HasAttribute("minValueBase")) MinValueBase = xml.GetDouble("minValueBase");
            if (xml.HasAttribute("maxValueBase")) MaxValueBase = xml.GetDouble("maxValueBase");

            if (xml.HasAttribute("k")) K = xml.GetDouble("k");
            Unit = xml.GetAttribute("unit");
            Format = xml.GetAttribute("format");

            if (xml.HasAttribute("modbusFunction")) this.ModbusReadFunction = (int)xml.GetInt("modbusFunction", 3);
            if (xml.HasAttribute("modbusReadFunction")) this.ModbusReadFunction = (int)xml.GetInt("modbusReadFunction", 3);
            if (xml.HasAttribute("modbusWriteFunction")) this.ModbusWriteFunction = (int)xml.GetInt("modbusWriteFunction", 6);
            if (xml.HasAttribute("readOnly")) ReadOnly = xml.GetAttribute("readOnly").StartsWith("t", StringComparison.InvariantCultureIgnoreCase);

            if (xml.HasAttribute("sign")) Sign = xml.GetAttribute("sign").StartsWith("t", StringComparison.InvariantCultureIgnoreCase);
            if (xml.HasAttribute("noWriteValue")) NoWriteValue = xml.GetInt("noWriteValue");
            if (xml.HasAttribute("groupCount")) GroupCount = (int)xml.GetInt("groupCount");

            MaxParamName = xml.GetAttribute("maxParam");
            MinParamName = xml.GetAttribute("minParam");
            KParamName = xml.GetAttribute("kParam");

            if (xml.HasAttribute("regNumber"))
            {
                this.ReadRegNumber = (int)xml.GetInt("regNumber", 0);
            }
            else
            {
                string[] tmp = Name.Split("-".ToCharArray(), 2, StringSplitOptions.None);
                if (tmp.Length == 2)
                {
                    if (int.TryParse(tmp[1].Trim(), NumberStyles.Integer, CultureInfo.InvariantCulture, out int ti2))
                    {
                        if (int.TryParse(tmp[0].Trim(), NumberStyles.HexNumber, CultureInfo.InvariantCulture, out int ti1))
                        {
                            this.ReadRegNumber = (ti1 << 8) + ti2;
                        }
                        else if (tmp[0] == "U0")
                        {
                            this.ReadRegNumber = 0x7000 + ti2;
                        }
                    }
                }
            }

            if (xml.HasAttribute("writeRegNumber"))
            {
                this.WriteRegNumber = (int)xml.GetInt("writeRegNumber", 0);
            }
            else
            {
                this.WriteRegNumber = ReadRegNumber;
            }

            if (xml.GetAttribute("valueType") == "list" ||
                xml.GetAttribute("valueType") == "list2" ||
                xml.GetAttribute("valueType") == "bitlist")
            {
                if (xml.GetAttribute("valueType") == "list") Type = ParamType.List;
                if (xml.GetAttribute("valueType") == "list2") Type = ParamType.List2;
                if (xml.GetAttribute("valueType") == "bitlist") Type = ParamType.BitList;

                foreach (XmlElement c in xml.GetElementsByTagName("item"))
                {
                    ParamValue v = new ParamValue();
                    //if (c.GetAttribute("value").Contains("Бит")) continue;
                    v.Value = c.GetInt("value");
                    v.Text = c.GetAttribute("text");
                    if (c.HasAttribute("k") || c.HasAttribute("K")) v.K = c.GetDouble("k");
                    if (c.HasAttribute("group")) v.Group = (int)c.GetInt("group", 1);
                    Values.Add(v);
                }
            }

            foreach (XmlElement x in xml.GetElementsByTagName("if"))
            {
                ParamInfoIf ifInfo = new ParamInfoIf();
                if (ifInfo.FromXml(x)) this.IfInfo.Add(ifInfo);
            }

            return true;
        }

        public int? GetValue(ComboBox control, bool move)
        {
            if (control.SelectedItem is ParamValue pv)
            {
                if (move) return (int)(pv.Value * pv.Group);
                return (int)pv.Value;
            }
            else if (control.SelectedItem is int iv)
            {
                return (int)iv;
            }
            else if (control.SelectedItem is short sv)
            {
                return (int)sv;
            }
            else if (control.SelectedItem is ushort usv)
            {
                return (int)usv;
            }
            else if (int.TryParse(control.Text, out int iv32))
            {
                return (int)iv32;
            }
            else
            {
                return null;
            }
        }

        bool _isUpdateControl = false;

        public void TextInputValue()
        {
            if (_isUpdateControl) return;

            if (ValueControlEditor != null)
            {
                /*
                string textValue = TextValueControl.Text;

                double dvValue = XmlHelper.StringToDouble(TextValueControl.Text);
                double outValue = K.HasValue ? outValue = dvValue / K.Value : dvValue;

                if (MinValue.HasValue && MinValue > dvValue ||
                    MaxValue.HasValue && MaxValue < dvValue)
                {
                    TextValueControl.Background = Brushes.Red;
                }
                else
                {
                    TextValueControl.Background = Brushes.White;
                }*/
            }
            else if (ComboBoxValueControlList != null && ComboBoxValueControlList.Count > 0)
            {
                int? value = 0;
                foreach (var c in ComboBoxValueControlList)
                {
                    int? tmp = GetValue(c, true);
                    if (tmp.HasValue)
                    {
                        if (!value.HasValue) value = 0;
                        value += tmp.Value;
                    }
                }
                ModbusValue = value;
            }
            else if (BitControl != null)
            {
                ModbusValue = BitControl.Value;
            }
            else if (ComboBoxValueControl != null)
            {
                ModbusValue = GetValue(ComboBoxValueControl, false);

                if (ComboBoxValueControl.SelectedItem == null)
                {
                    ComboBoxValueControl.Background = Brushes.Red;
                }
                else
                {
                    ComboBoxValueControl.Background = Brushes.White;
                }
            }
        }

        public void UpdateInputValue()
        {
            if (_isUpdateControl) return;

            unchecked
            {
                if (ValueControlEditor != null)
                {
                    /*
                    if (!ValueControlEditor.Value.HasValue)
                    {
                        ModbusValue = null;
                    }
                    else
                    {
                        double v = ValueControlEditor.Value.Value;
                        if (K.HasValue && K.Value != 0) v /= K.Value;
                        short dv = (short)Math.Round(v);
                        ModbusValue = (ushort)dv;
                    }*/
                }
                else if (ComboBoxValueControlList != null && ComboBoxValueControlList.Count > 0)
                {
                    int? value = 0;
                    foreach (var c in ComboBoxValueControlList)
                    {
                        int? tmp = GetValue(c, true);
                        if (tmp.HasValue)
                        {
                            if (!value.HasValue) value = 0;
                            value += tmp.Value;
                        }
                    }
                    ModbusValue = value;
                }
                else if (BitControl != null)
                {
                    ModbusValue = BitControl.Value;
                }
                else if (ComboBoxValueControl != null)
                {
                    var value = ComboBoxValueControl.SelectedItem as ParamValue;
                    if (value != null)
                    {
                        ModbusValue = (ushort)value.Value;
                    }
                    else if (int.TryParse(ComboBoxValueControl.Text, NumberStyles.Any, CultureInfo.InvariantCulture, out int dv))
                    {
                        ModbusValue = dv;
                    }
                    else
                    {
                        ModbusValue = null;
                    }
                }
            }
        }
    }


    public class ParamValue
    {
        public uint Value { get; set; }
        public int Group { get; set; } = 1;
        public string Text { get; set; }
        public double? K { get; set; }

        public override string ToString()
        {
            return $"{Value}:{Text}";
        }

        public override bool Equals(object obj)
        {
            if (obj is ParamValue v)
            {
                return this.Value == v.Value;
            }
            return false;
        }

        public override int GetHashCode()
        {
            return (int)Value;
        }
    }

    public class ParamGroup
    {
        public string Name { get; set; }
        DeviceConfig Device { get; set; }
        public List<ParamInfo> Params { get; set; } = new List<ParamInfo>();

        public bool Load(DeviceConfig device, XmlElement xml)
        {
            Device = device;
            Name = xml.GetAttribute("name");
            foreach (XmlElement pxml in xml.GetElementsByTagName("param"))
            {
                var pInfo = new ParamInfo();
                if (pInfo.Load(this, pxml))
                {
                    Params.Add(pInfo);
                    device.Params.Add(pInfo);
                }
            }
            return true;
        }
    }

    public enum ParamType
    {
        Value = 0,
        List = 1,
        List2 = 2,
        BitList = 3,
    }
}
