﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Win32;
using System.IO;

namespace FrequencyConverterMonitor
{
    /// <summary>
    /// Логика взаимодействия для FCConfigControl.xaml
    /// </summary>
    public partial class FCConfigControl : UserControl
    {
        DeviceInfo Device;
        public FCConfigControl()
        {
            InitializeComponent();
        }

        List<ParamInfo> _params = null;

        public void Init(DeviceInfo device)
        {
            Device = device;
            LeftPanel.Children.Clear();

            var bAll = new Button() { Content = "Все", Tag = Device.Config, Margin = new Thickness(3) };
            bAll.Click += GroupButton_Click;
            LeftPanel.Children.Add(bAll);

            _isUpdate = true;

            foreach (var g in Device.Config.Groups)
            {
                var bGroup = new Button() { Content = g.Name, Tag = g, Margin = new Thickness(3) };
                bGroup.Click += GroupButton_Click;
                LeftPanel.Children.Add(bGroup);
            }

            _params = device.Config.Params;
            _allReadCount = _params.Count;
            foreach (var pInfo in _params)
            {
                Border bName = new Border() { BorderThickness = new Thickness(1), BorderBrush = bBlack };
                Border bValue = new Border() { BorderThickness = new Thickness(1), BorderBrush = bBlack };

                pInfo.BorderCaptionControl = bName;
                pInfo.BorderValueControl = bValue;

                TextBlock tName = new TextBlock() { HorizontalAlignment = HorizontalAlignment.Left, VerticalAlignment = VerticalAlignment.Center, Margin = new Thickness(3) };
                tName.Text = pInfo.Name + ". " + pInfo.Caption;
                bName.Child = tName;

                if (pInfo.Type == ParamType.Value)
                {
                    var editor = new NumericUpDown();
                    editor.Tag = pInfo;
                    editor.LoadFromParamInfo(pInfo);
                    editor.ChangedValue += Editor_ChangedValue;
                    bValue.Child = editor;
                    pInfo.ValueControlEditor = editor;
                }
                else if (pInfo.Type == ParamType.List)
                {
                    var editor = new ComboBox();
                    foreach (var v in pInfo.Values)
                    {
                        editor.Items.Add(v);
                    }
                    if (editor.Items.Count > 0) editor.SelectedIndex = 0;
                    editor.Tag = pInfo;
                    editor.SelectionChanged += Editor_SelectionChanged;
                    editor.TextInput += Editor_TextInput;
                    editor.IsEditable = true;
                    editor.Margin = new Thickness(1);

                    bValue.Child = editor;
                    pInfo.ComboBoxValueControl = editor;
                }
                else if (pInfo.Type == ParamType.BitList)
                {
                    var editor = new BitComboBox();
                    foreach (var v in pInfo.Values)
                    {
                        editor.ItemsSource.Add(new SelectableObject(v));
                    }
                    editor.Tag = pInfo;
                    editor.SelectionChanged += Editor_SelectionChanged1; ;
                    editor.TextInput += Editor_TextInput;
                    editor.Margin = new Thickness(1);
                    bValue.Child = editor;
                    pInfo.BitControl = editor;
                }
                else if (pInfo.Type == ParamType.List2)
                {
                    for (int i = 0; i < pInfo.GroupCount; i++)
                    {
                        var editor1 = new ComboBox();
                        int b10 = (int)Math.Pow(10, i);

                        foreach (var v in pInfo.Values)
                        {
                            if (v.Group == b10) editor1.Items.Add(v);
                        }

                        if (editor1.Items.Count > 0) editor1.SelectedIndex = 0;

                        editor1.Tag = pInfo;
                        editor1.SelectionChanged += Editor_SelectionChanged;
                        editor1.TextInput += Editor_TextInput;
                        editor1.Margin = new Thickness(1);
                        //editor1.IsEditable = true;

                        if (pInfo.ComboBoxValueControlList == null) pInfo.ComboBoxValueControlList = new List<ComboBox>();
                        if (pInfo.ComboBoxValueControlList.Count == 0) pInfo.ComboBoxValueControl = editor1;
                        pInfo.ComboBoxValueControlList.Add(editor1);
                    }

                    StackPanel panel = new StackPanel();
                    panel.Orientation = Orientation.Vertical;
                    panel.VerticalAlignment = VerticalAlignment.Center;

                    foreach (var c in pInfo.ComboBoxValueControlList)
                    {
                        panel.Children.Add(c);
                    }

                    bValue.Child = panel;
                }
            }
            GroupButton_Click(null, null);

            _isUpdate = false;

        }

        private void Editor_SelectionChanged1(object sender, EventArgs e)
        {
            if (_isUpdate) return;
            try
            {
                _isUpdate = true;
                UpdateControls();
            }
            finally
            {
                _isUpdate = false;
            }
        }

        bool _isUpdate = false;

        public void UpdateControls()
        {
            foreach (var p in _params) p.UpdateParamInfo();
            foreach (var p in _params) p.UpdateParamInfo();

            foreach (var p in _params)
            {
                if (p.ValueControlEditor != null) p.ValueControlEditor.UpdateParamInfo(true);
            }
        }

        private void Editor_TextInput(object sender, TextCompositionEventArgs e)
        {
            if (_isUpdate) return;
            try
            {
                _isUpdate = true;
                ComboBox tb = (ComboBox)sender;
                ParamInfo pInfo = tb.Tag as ParamInfo;
                pInfo.TextInputValue();
                UpdateControls();
            }
            finally
            {
                _isUpdate = false;
            }
        }

        private void Editor_ChangedValue(object sender, EventArgs e)
        {
            if (_isUpdate) return;
            try
            {
                _isUpdate = true;
                UpdateControls();
            }
            finally
            {
                _isUpdate = false;
            }
        }

        private void Editor_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (_isUpdate) return;
            try
            {
                _isUpdate = true;
                ComboBox tb = (ComboBox)sender;
                ParamInfo pInfo = tb.Tag as ParamInfo;
                pInfo.TextInputValue();
                UpdateControls();
            }
            finally
            {
                _isUpdate = false;
            }
        }

        TextBox _lastEditTextBox = null;

        private void Editor_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox tb = (TextBox)sender;
            ParamInfo pInfo = tb.Tag as ParamInfo;
            pInfo.TextInputValue();
            _lastEditTextBox = tb;
        }

        enum WorkType
        {
            None,
            Read,
            ReadError,
            ReadEnd,
            Write,
            WriteError,
            WriteEnd,
        }

        volatile WorkType _isWork = WorkType.None;
        string _errorMessage = null;
        volatile int _readCount = 0;
        volatile int _allReadCount = 100;
        volatile bool _isClose = false;

        internal void OnTime()
        {
            lock (this)
            {
                if (_isWork == WorkType.ReadEnd || _isWork == WorkType.ReadError)
                {
                    gLoadGrid.Visibility = Visibility.Hidden;
                    if (_isWork == WorkType.ReadError)
                    {
                        gErrorGrid.Visibility = Visibility.Visible;
                        tErrorText.Text = "Ошибка загрузки конфигурации!\r\n" + _errorMessage;
                    }
                    _isWork = WorkType.None;
                    _errorMessage = string.Empty;

                    UpdateControls();
                }
                else if (_isWork == WorkType.WriteEnd || _isWork == WorkType.WriteError)
                {
                    gLoadGrid.Visibility = Visibility.Hidden;
                    if (_isWork == WorkType.WriteError)
                    {
                        gErrorGrid.Visibility = Visibility.Visible;
                        tErrorText.Text = "Ошибка записи конфигурации в прибор!\r\n" + _errorMessage;
                        _errorMessage = string.Empty;
                    }
                    _isWork = WorkType.None;
                }
                else if (_isWork == WorkType.Read || _isWork == WorkType.Write)
                {
                    pLoadProgressBar.Value = _readCount * 100 / _allReadCount;
                }

                if (_params != null)
                {
                    foreach (var v in _params)
                    {
                        if (v.IsChanged) v.UpdateControl();
                    }
                }
            }
        }


        public void ReadDeviceConfigs()
        {
            lock (this)
            {
                _isWork = WorkType.Read;
                _readCount = 0;
                pLoadProgressBar.Minimum = 0;
                pLoadProgressBar.Maximum = 100;
                pLoadProgressBar.Value = 0.0;
                tbLoadText.Text = "Загрузка конфигурации";
                gLoadGrid.Visibility = Visibility.Visible;
                gErrorGrid.Visibility = Visibility.Hidden;

                ThreadPool.QueueUserWorkItem(ReadRequest, this);
            }
        }

        protected void ReadRequest(object state)
        {
            try
            {
                int regError = 0;

                for (int i = 0; i < _allReadCount;)
                {
                    if (_isClose) return;
                    int count = 0;
                    int errorCount = 0;
                    do
                    {
                        count = Device.Update(_params, i);
                        if (count != 0) break;
                        errorCount++;
                    }
                    while (errorCount < 3);

                    if (count == 0)
                    {
                        _errorMessage += $"Ошибка чтения регистра '{_params[i].Name}\r\n";
                        Device.MainWindow.WriteToLog($"Ошибка чтения регистра '{_params[i].Name}' 0x{_params[i].ReadRegNumber.ToString("X4")}");

                        if (regError < 0)
                        {
                            regError++;
                            count = 1;
                        }
                        else
                        {
                            _isWork = WorkType.ReadError;
                            return;
                        }
                    }
                    
                    i += count;
                    _readCount += count;
                }
                _isWork = WorkType.ReadEnd;
            }
            catch
            {
                _isWork = WorkType.ReadError;
            }
            finally
            {
                if (_isWork != WorkType.ReadEnd) _isWork = WorkType.ReadError;
            }
        }

        public void WriteDeviceConfigs()
        {
            lock (this)
            {
                foreach (var p in _params) p.UpdateInputValue();

                _isWork = WorkType.Write;
                _readCount = 0;

                pLoadProgressBar.Minimum = 0;
                pLoadProgressBar.Maximum = 100;
                pLoadProgressBar.Value = 0.0;
                tbLoadText.Text = "Запись конфигурации";
                gLoadGrid.Visibility = Visibility.Visible;
                gErrorGrid.Visibility = Visibility.Hidden;

                ThreadPool.QueueUserWorkItem(WriteRequest, this);
            }
        }

        public void WriteRequest(object v)
        {
            try
            {
                foreach (var p in _params)
                {
                    if (_isClose) return;

                    if (p.ModbusValue.HasValue && !p.ReadOnly)
                    {
                        if (p.NoWriteValue.HasValue && p.ModbusValueU4 == p.NoWriteValue.Value) continue;

                        int e = 0;
                        int count = 0;
                        while (e < 3 && count == 0)
                        {
                            count = Device.WriteParam(p);
                            if (count == 0)
                            {
                                Device.MainWindow.WriteToLog($"Ошибка записи в регистр '{p.Name}' 0x{p.WriteRegNumber.ToString("X4")}");
                                e++;
                            }
                        }

                        if (count == 0)
                        {
                            _errorMessage += $"Ошибка записи в регистр'{p.Name}'\r\n";
                            _isWork = WorkType.WriteError;
                            return;
                        }
                    }
                    _readCount++;
                }
                _isWork = WorkType.WriteEnd;
            }
            catch
            {
                _isWork = WorkType.WriteError;
            }
            finally
            {
                if (_isWork != WorkType.WriteEnd) _isWork = WorkType.WriteError;
            }
        }

        public void SaveToFile()
        {
            try
            {
                SaveFileDialog dialog = new SaveFileDialog();
                dialog.Filter = "*.txt|*.txt|*.csv|*.csv|*.*|*.*";
                if (dialog.ShowDialog(this.Device.MainWindow) == true)
                {
                    string sep = dialog.FileName.EndsWith(".csv", StringComparison.InvariantCultureIgnoreCase) ? ";" : "\t";
                    using (StreamWriter writer = new StreamWriter(dialog.FileName, false, Encoding.UTF8))
                    {
                        writer.Write($"ParamName{sep}");
                        writer.Write($"RegNumber{sep}");
                        writer.Write($"HexValue{sep}");
                        writer.Write($"ParamCaption{sep}");
                        writer.WriteLine();

                        foreach (var p in _params)
                        {
                            p.UpdateInputValue();
                            writer.Write($"{p.Name}{sep}");
                            writer.Write($"{p.ReadRegNumber.ToString("X4")}{sep}");
                            writer.Write($"{p.ModbusValueU4?.ToString("X4")}{sep}");
                            writer.Write($"{p.Caption}{sep}");
                            writer.WriteLine();
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                System.Windows.MessageBox.Show(this.Device.MainWindow, "Ошибка при сохранение данных в файл:\r\n" + ex.ToString(), "Ошибка", MessageBoxButton.OK);
            }
        }

        public void LoadFromFile()
        {
            try
            {
                OpenFileDialog dialog = new OpenFileDialog();
                dialog.Filter = "*.txt|*.txt|*.csv|*.csv|*.*|*.*";
                if (dialog.ShowDialog(this.Device.MainWindow) == true)
                {
                    string sep = dialog.FileName.EndsWith(".csv", StringComparison.InvariantCultureIgnoreCase) ? ";" : "\t";
                    using (StreamReader reader = new StreamReader(dialog.FileName, Encoding.UTF8))
                    {
                        string line = reader.ReadLine();
                        int number = 0;

                        foreach (var p in _params) p.SetValue(null, 0);

                        while (!string.IsNullOrEmpty(line))
                        {
                            string[] data = line.Split(new string[] { sep }, 4, StringSplitOptions.None);
                            line = reader.ReadLine();

                            if (number == 0 && data.Length == 4 && data[1] == "RegNumber" || data.Length < 3) continue;

                            string pName = data[0];
                            int regNumber = int.Parse(data[1], System.Globalization.NumberStyles.HexNumber);
                            if (!string.IsNullOrEmpty(data[2]))
                            {
                                ushort regValue = ushort.Parse(data[2], System.Globalization.NumberStyles.HexNumber);
                                foreach (var p in _params)
                                {
                                    if (p.ReadRegNumber == regNumber && p.Name == pName)
                                    {
                                        p.SetValue(new ushort[] { regValue }, 0);
                                    }
                                }
                            }
                            number++;
                        }

                        UpdateControls();
                    }
                }
            }
            catch (Exception ex)
            {
                System.Windows.MessageBox.Show(this.Device.MainWindow, "Ошибка чтение данных из файла:\r\n" + ex.ToString(), "Ошибка", MessageBoxButton.OK);
            }
            finally
            {

            }
        }

        public void CloseTab()
        {
            _isClose = true;
            Device.CloseSettingsTab();
        }

        Brush bBlack = new SolidColorBrush(Color.FromRgb(0, 0, 0));

        private void GroupButton_Click(object sender, RoutedEventArgs e)
        {
            var button = sender as Button;
            List<ParamInfo> list = ((button?.Tag as ParamGroup)?.Params) ?? Device.Config.Params;

            ValueGrid.Children.Clear();
            ValueGrid.RowDefinitions.Clear();

            int row = 0;
            foreach (var pInfo in list)
            {
                if (pInfo.Type == ParamType.List2)
                {
                    ValueGrid.RowDefinitions.Add(new RowDefinition() { Height = new GridLength(28 * pInfo.GroupCount, GridUnitType.Pixel) });
                }
                else
                {
                    ValueGrid.RowDefinitions.Add(new RowDefinition() { Height = new GridLength(30, GridUnitType.Pixel) });
                }

                Border bName = pInfo.BorderCaptionControl;
                Border bValue = pInfo.BorderValueControl;

                Grid.SetColumn(bName, 0);
                Grid.SetRow(bName, row);
                ValueGrid.Children.Add(bName);

                Grid.SetColumn(bValue, 1);
                Grid.SetRow(bValue, row);
                ValueGrid.Children.Add(bValue);

                row++;
            }
        }

        private void bErrorOk_Click(object sender, RoutedEventArgs e)
        {
            gErrorGrid.Visibility = Visibility.Hidden;
        }
    }
}
