﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace FrequencyConverterMonitor
{
    /// <summary>
    /// Логика взаимодействия для FCSlide.xaml
    /// </summary>
    public partial class FCSlide : UserControl
    {
        public DeviceInfo Info { get; set; }

        public FCSlide()
        {
            InitializeComponent();
        }

        SolidColorBrush bFFA0C7EE = new SolidColorBrush(Color.FromRgb(0xA0, 0xC7, 0xEE));

        internal void UpdateState(bool active, ComminicationState state)
        {
            if (!active)
            {
                tbCaption.Text = Info.Name;
                tbErrorText.Text = "";
            }
            else if (state == ComminicationState.Normal || state == ComminicationState.None)
            {
                tbErrorText.Text = "";
            }
            else if (state == ComminicationState.ErrorResponse)
            {
                tbErrorText.Text = " - Неверный ответ";
            }
            else if (state == ComminicationState.Fault)
            {
                tbErrorText.Text = " - Ошибка";
            }
            else if (state == ComminicationState.NotResponse)
            {
                tbErrorText.Text = " - Нет ответа";
            }
            else if (state == ComminicationState.PortError)
            {
                tbErrorText.Text = " - Ошибка открытия порта";
            }
            UpdateErrorList();
        }

        public void Init(DeviceInfo info)
        {
            Info = info;
            tbCaption.Text = info.Name;
            cbActive.IsChecked = info.Active;

            gMainParams.Children.Clear();
            gMainParams.RowDefinitions.Clear();
            for (int i = 0; i< info.Config.MainParams.Count; i++)
            {
                SlideParam mParam = info.Config.MainParams[i];
                //<TextBlock Grid.Column="0" HorizontalAlignment="Center" VerticalAlignment="Center" FontWeight="Bold" Background="#FFA0C7EE">Заданная частота</TextBlock>
                //<TextBlock Grid.Column = "1" HorizontalAlignment = "Center" VerticalAlignment = "Center" FontWeight = "Bold"  Background = "#FFA0C7EE" > 100 </ TextBlock >
                mParam.txCaption = new TextBlock();
                mParam.txValue = new TextBlock();
                Border bTmp = new Border();

                bTmp.Cursor = Cursors.Hand;
                bTmp.Padding = new Thickness(1);

                bTmp.Tag = mParam;
                mParam.txValue.Tag = mParam;

                bTmp.Child = mParam.txValue;

                //bTmp.BorderBrush = Brushes.Black;
                //bTmp.BorderThickness = new Thickness(1);

                bTmp.MouseLeftButtonDown += TxValue_MouseLeftButtonDown;
                //mParam.txValue.MouseLeftButtonDown += TxValue_MouseLeftButtonDown;

                mParam.txCaption.HorizontalAlignment = HorizontalAlignment.Left;
                mParam.txValue.HorizontalAlignment = HorizontalAlignment.Center;

                mParam.txCaption.VerticalAlignment = VerticalAlignment.Center;
                mParam.txValue.VerticalAlignment = VerticalAlignment.Center;

                //mParam.txCaption.Background = bFFA0C7EE;
                //mParam.txValue.Background = bFFA0C7EE;

                mParam.txCaption.Text = mParam.Caption;
                mParam.txValue.Text = "????";

                Grid.SetColumn(mParam.txCaption, 0);
                Grid.SetColumn(bTmp, 1);
                Grid.SetRow(mParam.txCaption, i);
                Grid.SetRow(bTmp, i);

                gMainParams.RowDefinitions.Add(new RowDefinition() { Height = new GridLength(35, GridUnitType.Pixel) });

                gMainParams.Children.Add(mParam.txCaption);
                gMainParams.Children.Add(bTmp);
            }

            gSlideParams.Children.Clear();
            gSlideParams.RowDefinitions.Clear();

            for (int i = 0; i < info.Config.SlideParams.Count; i++)
            {
                SlideParam mParam = info.Config.SlideParams[i];
                //<TextBlock Grid.Row="0" Grid.Column="0" VerticalAlignment="Center">Параметр 1</TextBlock>
                //<TextBlock Grid.Row = "0" Grid.Column = "1" HorizontalAlignment = "Center" VerticalAlignment = "Center" > Значение 1 </ TextBlock >

                mParam.txCaption = new TextBlock();
                mParam.txValue = new TextBlock();

                //mParam.txCaption.HorizontalAlignment = HorizontalAlignment.Center;
                mParam.txValue.HorizontalAlignment = HorizontalAlignment.Center;

                mParam.txCaption.VerticalAlignment = VerticalAlignment.Center;
                mParam.txValue.VerticalAlignment = VerticalAlignment.Center;

                //mParam.txCaption.Background = bFFA0C7EE;
                //mParam.txValue.Background = bFFA0C7EE;

                mParam.txCaption.Margin = new Thickness(1, 1, 5, 1);
                mParam.txValue.Margin = new Thickness(1, 1, 5, 1);

                mParam.txCaption.Text = mParam.Caption;
                mParam.txValue.Text = "????";

                Grid.SetColumn(mParam.txCaption, 0);
                Grid.SetColumn(mParam.txValue, 1);
                Grid.SetRow(mParam.txCaption, i);
                Grid.SetRow(mParam.txValue, i);

                gSlideParams.RowDefinitions.Add(new RowDefinition() { Height = new GridLength(25, GridUnitType.Pixel) });
                gSlideParams.Children.Add(mParam.txCaption);
                gSlideParams.Children.Add(mParam.txValue);
            }

            gIn.Children.Clear();
            gIn.ColumnDefinitions.Clear();

            

            if (info.Config.Inputs.Count > 0)
            {
                Grid.SetColumn(bInBorder, 0);
                Grid.SetColumnSpan(bInBorder, info.Config.Inputs.Count);
                gIn.Children.Add(bInBorder);

                for (int i = 0; i < info.Config.Inputs.Count; i++)
                {
                    SlideParam mParam = info.Config.Inputs[i];
                    //<TextBlock Grid.Row="0" Grid.Column="0" VerticalAlignment="Center">Параметр 1</TextBlock>
                    //<TextBlock Grid.Row = "0" Grid.Column = "1" HorizontalAlignment = "Center" VerticalAlignment = "Center" > Значение 1 </ TextBlock >

                    mParam.txCaption = new TextBlock();
                    mParam.txValue = new TextBlock();

                    mParam.txCaption.HorizontalAlignment = HorizontalAlignment.Center;
                    mParam.txValue.HorizontalAlignment = HorizontalAlignment.Center;

                    mParam.txCaption.VerticalAlignment = VerticalAlignment.Center;
                    mParam.txValue.VerticalAlignment = VerticalAlignment.Center;

                    //mParam.txCaption.Background = bFFA0C7EE;
                    //mParam.txValue.Background = bFFA0C7EE;

                    mParam.txCaption.Text = mParam.Caption;
                    mParam.txValue.Text = "????";

                    var bCaption = ToBorder(mParam.txCaption);
                    var bValue = ToBorder(mParam.txValue);

                    bCaption.Padding = new Thickness(2);
                    bValue.Padding = new Thickness(2);

                    Grid.SetColumn(bCaption, i);
                    Grid.SetColumn(bValue, i);
                    Grid.SetRow(bCaption, 1);
                    Grid.SetRow(bValue, 2);

                    gIn.ColumnDefinitions.Add(new ColumnDefinition() { Width = new GridLength(1, GridUnitType.Star) });
                    gIn.Children.Add(bCaption);
                    gIn.Children.Add(bValue);
                }
            }

            gOut.Children.Clear();
            gOut.ColumnDefinitions.Clear();

            if (info.Config.Outputs.Count > 0)
            {
                gOut.Children.Add(bOutBorder);

                Grid.SetColumn(bOutBorder, 0);
                Grid.SetColumnSpan(bOutBorder, info.Config.Outputs.Count);

                for (int i = 0; i < info.Config.Outputs.Count; i++)
                {
                    SlideParam mParam = info.Config.Outputs[i];
                    //<TextBlock Grid.Row="0" Grid.Column="0" VerticalAlignment="Center">Параметр 1</TextBlock>
                    //<TextBlock Grid.Row = "0" Grid.Column = "1" HorizontalAlignment = "Center" VerticalAlignment = "Center" > Значение 1 </ TextBlock >

                    mParam.txCaption = new TextBlock();
                    mParam.txValue = new TextBlock();
                    mParam.txValue.Tag = mParam;

                    mParam.txCaption.HorizontalAlignment = HorizontalAlignment.Center;
                    mParam.txValue.HorizontalAlignment = HorizontalAlignment.Center;

                    mParam.txCaption.VerticalAlignment = VerticalAlignment.Center;
                    mParam.txValue.VerticalAlignment = VerticalAlignment.Center;

                    //mParam.txCaption.Background = bFFA0C7EE;
                    //mParam.txValue.Background = bFFA0C7EE;

                    mParam.txCaption.Text = mParam.Caption;
                    mParam.txValue.Text = "????";

                    var bCaption = ToBorder(mParam.txCaption);
                    var bValue = ToBorder(mParam.txValue);

                    bCaption.Padding = new Thickness(2);
                    bValue.Padding = new Thickness(2);

                    Grid.SetColumn(bCaption, i);
                    Grid.SetColumn(bValue, i);
                    Grid.SetRow(bCaption, 1);
                    Grid.SetRow(bValue, 2);

                    gOut.ColumnDefinitions.Add(new ColumnDefinition() { Width = new GridLength(1, GridUnitType.Star) });
                    gOut.Children.Add(bCaption);
                    gOut.Children.Add(bValue);
                }
            }
        }

        private void TxValue_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            FrameworkElement tb = sender as FrameworkElement;
            SlideParam mParam = tb.Tag as SlideParam;

            InputParamWindow dialog = new InputParamWindow();
            dialog.ShowDialog(this.Info, mParam);
        }

        List<TextBlock> errorList = new List<TextBlock>();
        int errorListCount = -1;

        public void UpdateErrorList()
        {
            
            int index = 0;

            foreach (var e in Info.Config.Errors)
            {
                if (e.Is)
                {
                    if (index == errorList.Count)
                    {
                        TextBlock tb = new TextBlock();
                        tb.Margin = new Thickness(3);
                        errorList.Add(tb);
                    }
                    if (errorList[index].Text != e.Text) errorList[index].Text = e.Text;
                    index++;
                }
            }

            if (errorListCount != index)
            {
                spErrorList.BeginInit();
                spErrorList.Children.Clear();
                spErrorList.Children.Add(tbErrorHeader);
                for (int i = 0; i < index; i++) spErrorList.Children.Add(errorList[i]);
                spErrorList.Children.Add(bResetError);
                spErrorList.EndInit();
                errorListCount = index;
            }
        }

        public TextBlock CreateTextBlock(string text)
        {
            TextBlock tb = new TextBlock();
            tb.Text = text;
            return tb;
        }

        public Border ToBorder(TextBlock tb)
        {
            Border b = new Border();
            b.Child = tb;
            b.BorderBrush = new SolidColorBrush(Colors.Black);
            b.BorderThickness = new Thickness(1);
            return b;
        }


        private void Button_Click(object sender, RoutedEventArgs e)
        {
            Info?.Close();
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            Info?.Hide();
        }

        private void Button_Click_2(object sender, RoutedEventArgs e)
        {
            Info.ShowSettings();
        }

        private void Button_Click_3(object sender, RoutedEventArgs e)
        {
            DeviceWindow dialog = new DeviceWindow();
            dialog.EditDevice(Info);
            Info.MainWindow.ProjectSave();
        }

        private void CheckBox_Checked(object sender, RoutedEventArgs e)
        {
            if (Info != null) Info.Active = cbActive.IsChecked ?? false;
        }

        private void cbActive_Unchecked(object sender, RoutedEventArgs e)
        {
            if (Info != null) Info.Active = cbActive.IsChecked ?? false;
        }

        private void Button_Click_4(object sender, RoutedEventArgs e)
        {
            //start
            Info.RunCmd(CommandType.Run);
        }

        private void Button_Click_5(object sender, RoutedEventArgs e)
        {
            //stop
            Info.RunCmd(CommandType.Stop);
        }

        private void Button_Click_6(object sender, RoutedEventArgs e)
        {
            //back
            Info.RunCmd(CommandType.Back);
        }

        private void Button_Click_7(object sender, RoutedEventArgs e)
        {
            //error reset
            Info.RunCmd(CommandType.ErrorReset);
        }
    }
}
