﻿using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xml;
using Microsoft.Win32;

namespace FrequencyConverterMonitor
{
    /// <summary>
    /// Логика взаимодействия для MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        string _filename = System.IO.Path.GetFullPath(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "FrequencyConverterMonitor.log"));
        string _comPortSetttingFileName = System.IO.Path.GetFullPath(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "FrequencyConverterMonitor.PortSettings.xml"));
        string _defaulProjectFileName = System.IO.Path.GetFullPath(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "FrequencyConverterMonitor.project.fcmprj"));

        public MainWindow()
        {
            InitializeComponent();

            dispatcherTimer = new System.Windows.Threading.DispatcherTimer();
            dispatcherTimer.Tick += DispatcherTimer_Tick;
            dispatcherTimer.Interval = new TimeSpan(0, 0, 0, 0, 250);
            dispatcherTimer.Start();

            try
            {
                if (File.Exists(_comPortSetttingFileName))
                {
                    XmlDocument doc = new XmlDocument();
                    doc.Load(_comPortSetttingFileName);
                    _portSettings.LoadFromXml(doc.DocumentElement);
                }
            }
            catch { }

            _comPort = new SerialPortContener(this, _portSettings);
            _workerThread = new System.Threading.Thread(Worker);
            _workerThread.Start();

            _project.Owner = this;
            _project.Load(_defaulProjectFileName);

            miFileLog.IsChecked = _portSettings.UseLog;
        }


        protected override void OnClosed(EventArgs e)
        {
            _exitEvent.Set();
            base.OnClosed(e);
            _project.Save();
        }

        private void DispatcherTimer_Tick(object sender, EventArgs e)
        {
            foreach (var d in _project.Devices) d.OnTimer();
            foreach(var t in MainTabControl.Items)
            {
                if (((TabItem)t).Content is FCConfigControl control)
                {
                    control.OnTime();
                }
            }

        }

        System.Windows.Threading.DispatcherTimer dispatcherTimer;
        System.Threading.Thread _workerThread = null;
        System.Threading.ManualResetEvent _exitEvent = new System.Threading.ManualResetEvent(false);
        SerialPortSettings _portSettings = new SerialPortSettings();
        ProjectInfo _project = new ProjectInfo();

        public bool IsSimMode { get; set; } = false;

        int deviceNumber = 0;
        private void MenuItem_Click(object sender, RoutedEventArgs e)
        {
            DeviceWindow dialog = new DeviceWindow();

            var device = dialog.CreateDevice(this, "ПЧ #" + (++deviceNumber));
            if (device != null)
            {
                device.Show();
                _project.Add(device);
                _project.Save();
            }
        }

        public void DeleteDevice(DeviceInfo info)
        {
            _project.Remove(info);
            if (info.ShowButton != null && info.ShowButton.Parent != null) SleepPanel.Children.Remove(info.ShowButton);
            if (info.SlideControl != null && info.SlideControl.Parent != null) MainSlidePanel.Children.Remove(info.SlideControl);
            if (info.SettingTab != null && info.SettingTab.Parent != null) MainTabControl.Items.Remove(info.SettingTab);
            _project.Save();
        }

        public void ProjectSave()
        {
            _project.Save();
        }

        private void MenuItem_Click_1(object sender, RoutedEventArgs e)
        {
            ComPortWindow comsettings = new ComPortWindow();

            if (comsettings.ShowDialog(this, _portSettings) == true)
            {
                SavePortSetting();
            }
        }

        private void SavePortSetting()
        {
            try
            {
                XmlDocument doc = new XmlDocument();
                doc.LoadXml("<?xml version=\"1.0\" encoding=\"utf-8\" ?>\r\n<portsettings/>");
                _portSettings.SaveToXml(doc.DocumentElement);
                _comPort.SetSettings(_portSettings);
                doc.Save(_comPortSetttingFileName);
            }
            catch { }
        }

        public void WriteOpenPort(SerialPort port)
        {
            try
            {
                if (_portSettings.UseLog)
                {
                    using (StreamWriter sw = new StreamWriter(_filename, true, Encoding.UTF8))
                    {
                        sw.WriteLine("[{0:yyyy-MM-dd HH:mm:ss.fff}] OPEN('{1}', BaudRate:{2}, Parity:{3}, StopBits:{6}, DataBits:{4} Handshake:{5})", DateTime.Now, port.PortName, port.BaudRate, port.Parity, port.DataBits, port.Handshake, port.StopBits);
                    }
                }
            }
            catch (Exception) { }
        }

        public void WriteToLog(bool bIn, byte[] data, int leng)
        {
            try
            {
                if (_portSettings.UseLog)
                {
                    using (StreamWriter sw = new StreamWriter(_filename, true, Encoding.UTF8))
                    {
                        sw.WriteLine("[{0:yyyy-MM-dd HH:mm:ss.fff}] {1} {2}", DateTime.Now, bIn ? "<<" : ">>", BytesToString(data, leng));
                    }
                }
            }
            catch (Exception) { }
        }

        public void WriteToLog(string message)
        {
            try
            {
                if (_portSettings.UseLog)
                {
                    using (StreamWriter sw = new StreamWriter(_filename, true, Encoding.UTF8))
                    {
                        sw.WriteLine("[{0:yyyy-MM-dd HH:mm:ss.fff}] {1}", DateTime.Now, message);
                    }
                }
            }
            catch (Exception) { }
        }

        public void WriteToLog(Exception ex)
        {
            try
            {
                if (_portSettings.UseLog)
                {
                    using (StreamWriter sw = new StreamWriter(_filename, true, Encoding.UTF8))
                    {
                        sw.WriteLine("[{0:yyyy-MM-dd HH:mm:ss.fff}] Ошибка: {1}", DateTime.Now, ex);
                    }
                }
            }
            catch (Exception) { }
        }

       

        public static string BytesToString(byte[] data, int leng)
        {
            StringBuilder str = new StringBuilder();
            if (data == null) return null;
            for (int i = 0; i < leng; i++)
            {
                byte b = data[i];
                str.Append(b.ToString("X2"));
            }
            return str.ToString();
        }

        SerialPortContener _comPort = null;

        public void RunCmd(DeviceInfo info, CommandType cmd)
        {
            try
            {
                bool ret;
                lock (_comPort)
                {
                    _comPort.TryOpen();
                    ret = info.RunCmd(_comPort, cmd);
                }
                if (!ret) MessageBox.Show(this, "Не удалось записать значение в прибор!", info.Name, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch
            {
                MessageBox.Show(this, "Ошибка записи значения в прибор!", info.Name, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        internal int Update(DeviceInfo deivce, List<ParamInfo> plist, int index)
        {
            return deivce.Update(_exitEvent, _comPort, plist, index);
        }

        internal int WriteParam(DeviceInfo deviceInfo, ParamInfo pInfo)
        {
            return deviceInfo.WriteParam(_exitEvent, _comPort, pInfo);
        }

        public bool WriteParam(DeviceInfo deviceInfo, int f, int reg, ushort data)
        {
            return deviceInfo.WriteParam(_exitEvent, _comPort, f, reg, data);
        }

        private void Worker()
        {
            List<DeviceInfo> devices = new List<DeviceInfo>();

            while (!_exitEvent.WaitOne(100))
            {
                try
                {
                    devices.Clear();
                    foreach (var d in _project.Devices) if (d.Active && d.IsShow) devices.Add(d);

                    if (devices.Count > 0 && IsSimMode)
                    {
                        foreach (var d in devices) d.SimUpdate();
                    }
                    else if (devices.Count > 0 && !string.IsNullOrEmpty(_portSettings.Port))
                    {
                        try
                        {
                            foreach (var d in devices)
                            {
                                d.Update(_exitEvent, _comPort);
                            }
                        }
                        catch
                        {
                            continue;
                        }
                        finally
                        {
                        }
                    }
                }
                catch { }
            }
        }

       

        private void miExit_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void miCreateProject_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.DefaultExt = ".fcmprj";
            dialog.Filter = "*.fcmprj|*.fcmprj";
            if (dialog.ShowDialog() == true)
            {
                //save
                _project.FileName = dialog.FileName;
                _project.Clear();
                _project.Save();
            }
        }

        private void miOpenProject_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.DefaultExt = ".fcmprj";
            dialog.Filter = "*.fcmprj|*.fcmprj";

            if (dialog.ShowDialog() == true)
            {
                _project.Load(dialog.FileName);
            }
        }

        private void miSaveProject_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.DefaultExt = ".fcmprj";
            dialog.Filter = "*.fcmprj|*.fcmprj";

            if (dialog.ShowDialog() == true)
            {
                //save
                _project.SaveAs(dialog.FileName);
            }
        }

        private void miSimMode_Click(object sender, RoutedEventArgs e)
        {
            IsSimMode = !IsSimMode;
            miSimMode.IsChecked = IsSimMode;
        }

        private void miFileLog_Click(object sender, RoutedEventArgs e)
        {
            miFileLog.IsChecked = !miFileLog.IsChecked;
            _portSettings.UseLog = miFileLog.IsChecked;
            SavePortSetting();
        }

        private void miCleareProject_Click(object sender, RoutedEventArgs e)
        {
            _project.Clear();
            _project.Save();
        }

        private void MainTabControl_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            TabItem tab = (TabItem)MainTabControl.SelectedItem;
            if (tab?.Content is FCConfigControl)
            {
                if (FCSettings.Visibility != Visibility.Visible)
                {
                    FCSettings.Visibility = Visibility.Visible;
                    mMenu.Items.Insert(2, FCSettings);
                }
            }
            else
            {
                FCSettings.Visibility = Visibility.Hidden;
                mMenu.Items.Remove(FCSettings);
            }
        }

        private void miFSClose_Click(object sender, RoutedEventArgs e)
        {
            if (((TabItem)MainTabControl.SelectedItem).Content is FCConfigControl control)
            {
                control.CloseTab();
            }
        }

        private void miFSOpen_Click(object sender, RoutedEventArgs e)
        {
            if (((TabItem)MainTabControl.SelectedItem).Content is FCConfigControl control)
            {
                control.LoadFromFile();
            }
        }

        private void miFSSave_Click(object sender, RoutedEventArgs e)
        {
            if (((TabItem)MainTabControl.SelectedItem).Content is FCConfigControl control)
            {
                control.SaveToFile();
            }
        }

        private void miFSWrite_Click(object sender, RoutedEventArgs e)
        {
            if (((TabItem)MainTabControl.SelectedItem).Content is FCConfigControl control)
            {
                control.WriteDeviceConfigs();
            }
        }

        private void miFSRead_Click(object sender, RoutedEventArgs e)
        {
            if (((TabItem)MainTabControl.SelectedItem).Content is FCConfigControl control)
            {
                control.ReadDeviceConfigs();
            }
        }
    }
}
