﻿using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Linq;
using System.Text;

namespace FrequencyConverterMonitor
{
    public class SerialPortContener: IDisposable
    {
        protected SerialPort _port;
        protected SerialPortSettings _settings;
        protected bool _reset = false;
        protected bool _isOpen = false;
        protected MainWindow _main;

        public SerialPortContener(MainWindow main)
        {
            _main = main;
        }

        public SerialPortContener(MainWindow main, SerialPortSettings settings)
        {
            _main = main;
            _settings = settings.Clone();
        }

        public SerialPortContener(SerialPort port)
        {
            _port = port;
        }

        public void SetSettings(SerialPortSettings settings)
        {
            lock (this)
            {
                _settings = settings.Clone();
                _reset = true;
            }
        }

        private void InternalOpen()
        {
            if (_reset && _port != null)
            {
                _port.Close();
            }
            _reset = false;

            if (_port == null)
            {
                try
                {
                    _port = new SerialPort(_settings.Port, _settings.Speed, _settings.Parity, _settings.DataBits, _settings.StopBits);
                    _port.ReadTimeout = 2000;
                    _port.WriteTimeout = 2000;
                    _port.Open();
                    _main.WriteOpenPort(_port);
                    _isOpen = true;
                }
                catch
                {
                    _isOpen = false;
                    try { _port.Dispose(); } catch { };
                    _port = null;
                    throw;
                }
            }
        }

        private void InternalClose()
        {
            lock (this)
            {
                _isOpen = false;
                try { _port.Dispose(); } catch { };
                _port = null;
            }
        }

        public void Open()
        {
            
        }

        public void Close()
        {

        }

        public int Read(byte[] buffer, int offset, int count)
        {
            lock (this)
            {
                try
                {
                    InternalOpen();
                    return _port.Read(buffer, offset, count);
                }
                catch
                {
                    InternalClose();
                    throw;
                }
            }
        }

        public void Write(byte[] buffer, int offset, int count)
        {
            lock (this)
            {
                try
                {
                    InternalOpen();
                    _port.Write(buffer, offset, count);
                }
                catch
                {
                    InternalClose();
                    throw;
                }
            }
        }

        public void DiscardInBuffer()
        {
            lock (this)
            {
                try
                {
                    InternalOpen();
                    _port.DiscardInBuffer();
                }
                catch
                {
                    InternalClose();
                    throw;
                }
            }
        }


        

        public void Dispose()
        {
            lock (this)
            {
                if (_port != null)
                {
                    _port.Close();
                    _port.Dispose();
                }
                _isOpen = false;
            }
        }

        internal void TryOpen()
        {
            lock (this)
            {
                InternalOpen();
            }
        }

    }
}
