﻿using System;
using System.IO.Ports;
using System.Xml;
using System.Xml.Serialization;

namespace FrequencyConverterMonitor
{
    public class SerialPortSettings
    {
        string _host = "COM1";
        Parity _parity = Parity.None;
        Handshake _handShake = Handshake.None;
        int _speed = 9600;
        StopBits _stopBits = StopBits.Two;
        int _dataBits = 8;
        bool _useLog = false;

        public SerialPortSettings Clone()
        {
            SerialPortSettings item = new SerialPortSettings();
            item.Port = this.Port;
            item.Speed = this.Speed;
            item.HandShake = this.HandShake;
            item.Parity = this.Parity;
            item.StopBits = this.StopBits;
            item.DataBits = this.DataBits;
            return item;
        }

        public void SaveToXml(XmlElement xml)
        {
            xml.SetAttribute("Port", Port);
            xml.SetAttribute("Speed", Speed.ToString());
            xml.SetAttribute("HandShake", HandShake.ToString());
            xml.SetAttribute("Parity", Parity.ToString());
            xml.SetAttribute("StopBits", StopBits.ToString());
            xml.SetAttribute("DataBits", DataBits.ToString());
            xml.SetAttribute("UseLog", UseLog.ToString());
        }

        public void LoadFromXml(XmlElement xml)
        {
            try { Port = xml.GetAttribute("Port"); }
            catch { }
            try { Speed = int.Parse(xml.GetAttribute("Speed")); }
            catch { }
            try { HandShake = (Handshake)Enum.Parse(typeof(Handshake), xml.GetAttribute("HandShake")); }
            catch { }
            try { Parity = (Parity)Enum.Parse(typeof(Parity), xml.GetAttribute("Parity")); }
            catch { }
            try { StopBits = (StopBits)Enum.Parse(typeof(StopBits), xml.GetAttribute("StopBits")); }
            catch { }
            try { DataBits = int.Parse(xml.GetAttribute("DataBits")); }
            catch { }
            try { UseLog = bool.Parse(xml.GetAttribute("UseLog")); }
            catch { }
        }

        [XmlAttribute]
        public bool UseLog
        {
            get { return _useLog; }
            set { _useLog = value; }
        }

        [XmlAttribute]
        public string Port
        {
            get { return _host; }
            set { _host = value; }
        }


        [XmlAttribute]
        public int Speed
        {
            get { return _speed; }
            set { _speed = value; }
        }


        [XmlAttribute]
        public Parity Parity
        {
            get { return _parity; }
            set { _parity = value; }
        }


        [XmlAttribute]
        public Handshake HandShake
        {
            get { return _handShake; }
            set { _handShake = value; }
        }


        [XmlAttribute]
        public StopBits StopBits
        {
            get { return _stopBits; }
            set { _stopBits = value; }
        }


        [XmlAttribute]
        public int DataBits
        {
            get { return _dataBits; }
            set { _dataBits = value; }
        }

        public static SerialPortSettings Default
        {
            get
            {
                SerialPortSettings res = new SerialPortSettings();
                res._dataBits = 8;
                res._handShake = Handshake.None;
                res._host = "COM1";
                res._parity = Parity.None;
                res._speed = 9600;
                res._stopBits = StopBits.Two;

                return res;
            }
        }
    }
}
