﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;

namespace FrequencyConverterMonitor
{
    public static class XmlHelper
    {
        public static uint GetInt(this XmlElement xml, string name)
        {
            string v = xml.GetAttribute(name);
            return StringToInt(v);
        }

        public static uint GetInt(this XmlElement xml, string name, uint defaultValue)
        {
            string v = xml.GetAttribute(name);
            if (string.IsNullOrWhiteSpace(v)) return defaultValue;
            return StringToInt(v);
        }

        public static double GetDouble(this XmlElement xml, string name)
        {
            string v = xml.GetAttribute(name);
            if (v == null) v = xml.GetAttribute(name.ToUpper());
            if (v == null) v = xml.GetAttribute(name.ToLower());
            return StringToDouble(v);
        }


        public static uint StringToInt(string v)
        {
            if (v.StartsWith("0x", StringComparison.InvariantCultureIgnoreCase))
            {
                return uint.Parse(v.Substring(2), System.Globalization.NumberStyles.HexNumber);
            }
            else if (v.StartsWith("h", StringComparison.InvariantCultureIgnoreCase))
            {
                return uint.Parse(v.Substring(1), System.Globalization.NumberStyles.HexNumber);
            }
            else if (v.EndsWith("H", StringComparison.InvariantCultureIgnoreCase))
            {
                return uint.Parse(v.Substring(0, v.Length - 1), System.Globalization.NumberStyles.HexNumber);
            }
            else if (string.IsNullOrEmpty(v))
            {
                return 0;
            }
            else
            {
                return uint.Parse(v);
            }
        }

        public static double StringToDouble(string v)
        {
            StringBuilder tmp = new StringBuilder();
            foreach (var c in v.Trim())
            {
                if (char.IsDigit(c) || c == '-') tmp.Append(c);
                else if (c == ',' || c == '.') tmp.Append('.');
                else break;
            }
            return double.Parse(tmp.ToString(), System.Globalization.CultureInfo.InvariantCulture);
        }
    }
}
